@@ -1,6 +1,8 @@
 """Provide a model for the Z-Wave JS controller."""
-from typing import TYPE_CHECKING, Dict, List, Optional, TypedDict, cast
+from dataclasses import dataclass
+from typing import TYPE_CHECKING, Dict, List, Literal, Optional, TypedDict, Union, cast
 
+from ..const import InclusionStrategy, SecurityClass
 from ..event import Event, EventBase
 from .association import Association, AssociationGroup
 from .node import Node
@@ -9,8 +11,121 @@ if TYPE_CHECKING:
     from ..client import Client
 
 
+class InclusionGrantDataType(TypedDict):
+    """Representation of an inclusion grant data dict type."""
+
+    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/Inclusion.ts#L48-L56
+    securityClasses: List[SecurityClass]
+    clientSideAuth: bool
+
+
+@dataclass
+class InclusionGrant:
+    """Representation of an inclusion grant."""
+
+    security_classes: List[SecurityClass]
+    client_side_auth: bool
+
+    def to_dict(self) -> InclusionGrantDataType:
+        """Return InclusionGrantDataType dict from self."""
+        return {
+            "securityClasses": self.security_classes,
+            "clientSideAuth": self.client_side_auth,
+        }
+
+    @classmethod
+    def from_dict(cls, data: InclusionGrantDataType) -> "InclusionGrant":
+        """Return InclusionGrant from InclusionGrantDataType dict."""
+        return cls(
+            [SecurityClass(sec_cls) for sec_cls in data["securityClasses"]],
+            data["clientSideAuth"],
+        )
+
+
+class ControllerStatisticsDataType(TypedDict):
+    """Represent a controller statistics data dict type."""
+
+    # https://github.com/zwave-js/node-zwave-js/blob/master/packages/zwave-js/src/lib/controller/ControllerStatistics.ts#L20-L39
+    messagesTX: int
+    messagesRX: int
+    messagesDroppedTX: int
+    messagesDroppedRX: int
+    NAK: int
+    CAN: int
+    timeoutACK: int
+    timeoutResponse: int
+    timeoutCallback: int
+
+
+class ControllerStatistics:
+    """Represent a controller statistics update."""
+
+    def __init__(self, data: Optional[ControllerStatisticsDataType] = None) -> None:
+        """Initialize controller statistics."""
+        self.data = data or ControllerStatisticsDataType(
+            CAN=0,
+            messagesDroppedRX=0,
+            messagesDroppedTX=0,
+            messagesRX=0,
+            messagesTX=0,
+            NAK=0,
+            timeoutACK=0,
+            timeoutCallback=0,
+            timeoutResponse=0,
+        )
+
+    @property
+    def messages_tx(self) -> int:
+        """Return number of messages successfully sent to controller."""
+        return self.data["messagesTX"]
+
+    @property
+    def messages_rx(self) -> int:
+        """Return number of messages received by controller."""
+        return self.data["messagesRX"]
+
+    @property
+    def messages_dropped_rx(self) -> int:
+        """Return number of messages from controller that were dropped by host."""
+        return self.data["messagesDroppedRX"]
+
+    @property
+    def messages_dropped_tx(self) -> int:
+        """
+        Return number of outgoing messages that were dropped.
+
+        These messages could not be sent.
+        """
+        return self.data["messagesDroppedTX"]
+
+    @property
+    def nak(self) -> int:
+        """Return number of messages that controller did not accept."""
+        return self.data["NAK"]
+
+    @property
+    def can(self) -> int:
+        """Return number of collisions while sending a message to controller."""
+        return self.data["CAN"]
+
+    @property
+    def timeout_ack(self) -> int:
+        """Return number of transmission attempts without an ACK from controller."""
+        return self.data["timeoutACK"]
+
+    @property
+    def timeout_response(self) -> int:
+        """Return number of transmission attempts where controller response timed out."""
+        return self.data["timeoutResponse"]
+
+    @property
+    def timeout_callback(self) -> int:
+        """Return number of transmission attempts where controller callback timed out."""
+        return self.data["timeoutCallback"]
+
+
 class ControllerDataType(TypedDict, total=False):
-    """Represent a node data dict type."""
+    """Represent a controller data dict type."""
 
     libraryVersion: str
     type: int
@@ -30,6 +145,7 @@ class ControllerDataType(TypedDict, total=False):
     sucNodeId: int
     supportsTimers: bool
     isHealNetworkActive: bool
+    statistics: ControllerStatisticsDataType
 
 
 class Controller(EventBase):
@@ -40,7 +156,9 @@ class Controller(EventBase):
         super().__init__()
         self.client = client
         self.data: ControllerDataType = state["controller"]
+        self._statistics = ControllerStatistics(self.data.get("statistics"))
         self.nodes: Dict[int, Node] = {}
+        self._heal_network_progress: Optional[Dict[int, str]] = None
         for node_state in state["nodes"]:
             node = Node(client, node_state)
             self.nodes[node.node_id] = node
@@ -145,19 +263,52 @@ class Controller(EventBase):
         return self.data.get("supportsTimers")
 
     @property
-    def is_heal_network_active(self) -> bool:
+    def is_heal_network_active(self) -> Optional[bool]:
         """Return is_heal_network_active."""
-        return self.data["isHealNetworkActive"]
+        return self.data.get("isHealNetworkActive")
+
+    @property
+    def statistics(self) -> ControllerStatistics:
+        """Return statistics property."""
+        return self._statistics
+
+    @property
+    def heal_network_progress(self) -> Optional[Dict[int, str]]:
+        """Return heal network progress state."""
+        return self._heal_network_progress
 
     async def async_begin_inclusion(
-        self, include_non_secure: Optional[bool] = None
+        self,
+        inclusion_strategy: Union[
+            Literal[InclusionStrategy.SECURITY_S0],
+            Literal[InclusionStrategy.SECURITY_S2],
+            Literal[InclusionStrategy.INSECURE],
+        ],
     ) -> bool:
         """Send beginInclusion command to Controller."""
         data = await self.client.async_send_command(
             {
                 "command": "controller.begin_inclusion",
-                "includeNonSecure": include_non_secure,
-            }
+                "options": {"strategy": inclusion_strategy},
+            },
+            require_schema=8,
+        )
+        return cast(bool, data["success"])
+
+    async def async_begin_inclusion_default(
+        self,
+        force_security: Optional[bool] = None,
+    ) -> bool:
+        """Send beginInclusion command to Controller using Default inclusion method."""
+        data = await self.client.async_send_command(
+            {
+                "command": "controller.begin_inclusion",
+                "options": {
+                    "strategy": InclusionStrategy.DEFAULT,
+                    "forceSecurity": force_security,
+                },
+            },
+            require_schema=8,
         )
         return cast(bool, data["success"])
 
@@ -189,15 +340,22 @@ class Controller(EventBase):
         )
 
     async def async_replace_failed_node(
-        self, node_id: int, include_non_secure: Optional[bool] = None
+        self,
+        node_id: int,
+        inclusion_strategy: Union[
+            Literal[InclusionStrategy.SECURITY_S0],
+            Literal[InclusionStrategy.SECURITY_S2],
+            Literal[InclusionStrategy.INSECURE],
+        ],
     ) -> bool:
         """Send replaceFailedNode command to Controller."""
         data = await self.client.async_send_command(
             {
                 "command": "controller.replace_failed_node",
                 "nodeId": node_id,
-                "includeNonSecure": include_non_secure,
-            }
+                "options": {"strategy": inclusion_strategy},
+            },
+            require_schema=8,
         )
         return cast(bool, data["success"])
 
@@ -220,7 +378,11 @@ class Controller(EventBase):
         data = await self.client.async_send_command(
             {"command": "controller.stop_healing_network"}
         )
-        return cast(bool, data["success"])
+        success = cast(bool, data["success"])
+        if success:
+            self._heal_network_progress = None
+            self.data["isHealNetworkActive"] = False
+        return success
 
     async def async_is_failed_node(self, node_id: int) -> bool:
         """Send isFailedNode command to Controller."""
@@ -340,6 +502,26 @@ class Controller(EventBase):
         )
         return cast(List[int], data["neighbors"])
 
+    async def async_grant_security_classes(
+        self, inclusion_grant: InclusionGrant
+    ) -> None:
+        """Send grantSecurityClasses command to Controller."""
+        await self.client.async_send_command(
+            {
+                "command": "controller.grant_security_classes",
+                "inclusionGrant": inclusion_grant.to_dict(),
+            }
+        )
+
+    async def async_validate_dsk_and_enter_pin(self, pin: str) -> None:
+        """Send validateDSKAndEnterPIN command to Controller."""
+        await self.client.async_send_command(
+            {
+                "command": "controller.validate_dsk_and_enter_pin",
+                "pin": pin,
+            }
+        )
+
     def receive_event(self, event: Event) -> None:
         """Receive an event."""
         if event.data["source"] == "node":
@@ -391,10 +573,25 @@ class Controller(EventBase):
 
     def handle_heal_network_progress(self, event: Event) -> None:
         """Process a heal network progress event."""
-        # pylint: disable=unused-argument
+        self._heal_network_progress = event.data["progress"].copy()
         self.data["isHealNetworkActive"] = True
 
     def handle_heal_network_done(self, event: Event) -> None:
         """Process a heal network done event."""
         # pylint: disable=unused-argument
+        self._heal_network_progress = None
         self.data["isHealNetworkActive"] = False
+
+    def handle_statistics_updated(self, event: Event) -> None:
+        """Process a statistics updated event."""
+        self._statistics.data.update(event.data["statistics"])
+        event.data["statistics_updated"] = self.statistics
+
+    def handle_grant_security_classes(self, event: Event) -> None:
+        """Process a grant security classes event."""
+        event.data["requested_grant"] = InclusionGrant.from_dict(
+            event.data["requested"]
+        )
+
+    def handle_validate_dsk_and_enter_pin(self, event: Event) -> None:
+        """Process a validate dsk and enter pin event."""
